@@ -23,16 +23,18 @@ mod common;
 use std::collections::{HashMap, HashSet};
 use std::time::SystemTime;
 
-use common::{NodeFixture, TestNodeWriter};
+use common::{NodeFixture, TestNodeReader, TestNodeWriter};
 use nucliadb_core::protos::op_status::Status;
 use nucliadb_core::protos::prost_types::Timestamp;
 use nucliadb_core::protos::relation::RelationType;
 use nucliadb_core::protos::relation_node::NodeType;
 use nucliadb_core::protos::resource::ResourceStatus;
 use nucliadb_core::protos::{
-    EntitiesSubgraphRequest, IndexMetadata, NewShardRequest, Relation, RelationNode, RelationNodeFilter,
-    RelationPrefixSearchRequest, RelationSearchRequest, RelationSearchResponse, Resource, ResourceId,
+    EntitiesSubgraphRequest, IndexMetadata, NewShardRequest, Relation, RelationMetadata, RelationNode,
+    RelationNodeFilter, RelationPrefixSearchRequest, RelationSearchRequest, RelationSearchResponse, Resource,
+    ResourceId,
 };
+use nucliadb_protos::nodereader::SearchRequest;
 use rstest::*;
 use tonic::Request;
 use uuid::Uuid;
@@ -259,6 +261,15 @@ async fn create_knowledge_graph(writer: &mut TestNodeWriter, shard_id: String) -
             source: Some(relation_nodes.get("Poetry").unwrap().clone()),
             to: Some(relation_nodes.get("Swallow").unwrap().clone()),
             relation_label: "about".to_string(),
+            metadata: Some(RelationMetadata {
+                paragraph_id: Some("myresource/0/myresource/100-200".to_string()),
+                source_start: Some(0),
+                source_end: Some(10),
+                to_start: Some(11),
+                to_end: Some(20),
+                data_augmentation_task_id: Some("mytask".to_string()),
+                ..Default::default()
+            }),
             ..Default::default()
         },
         Relation {
@@ -338,19 +349,21 @@ async fn test_search_relations_prefixed() -> Result<(), Box<dyn std::error::Erro
     // Test: prefixed search with empty term. Results are limited
     // --------------------------------------------------------------
 
-    let response = reader
-        .relation_search(RelationSearchRequest {
+    let response = relation_search(
+        &mut reader,
+        RelationSearchRequest {
             shard_id: shard_id.clone(),
             prefix: Some(RelationPrefixSearchRequest {
                 prefix: String::new(),
                 ..Default::default()
             }),
             ..Default::default()
-        })
-        .await?;
+        },
+    )
+    .await?;
 
-    assert!(response.get_ref().prefix.is_some());
-    let prefix_response = response.get_ref().prefix.as_ref().unwrap();
+    assert!(response.prefix.is_some());
+    let prefix_response = response.prefix.as_ref().unwrap();
     let results = &prefix_response.nodes;
     // TODO this constants is spread between relations and paragraphs. It should
     // be in a single place and common for everyone
@@ -361,8 +374,9 @@ async fn test_search_relations_prefixed() -> Result<(), Box<dyn std::error::Erro
     // Test: prefixed search with "cat" term (some results)
     // --------------------------------------------------------------
 
-    let response = reader
-        .relation_search(RelationSearchRequest {
+    let response = relation_search(
+        &mut reader,
+        RelationSearchRequest {
             shard_id: shard_id.clone(),
             prefix: Some(RelationPrefixSearchRequest {
                 prefix: "cat".to_string(),
@@ -372,12 +386,13 @@ async fn test_search_relations_prefixed() -> Result<(), Box<dyn std::error::Erro
                 }],
             }),
             ..Default::default()
-        })
-        .await?;
+        },
+    )
+    .await?;
 
     let expected = HashSet::from_iter(["Cat".to_string(), "Catwoman".to_string(), "Batman".to_string()]);
-    assert!(response.get_ref().prefix.is_some());
-    let prefix_response = response.get_ref().prefix.as_ref().unwrap();
+    assert!(response.prefix.is_some());
+    let prefix_response = response.prefix.as_ref().unwrap();
     let results = prefix_response.nodes.iter().map(|node| node.value.to_owned()).collect::<HashSet<_>>();
     assert_eq!(results, expected);
 
@@ -385,8 +400,9 @@ async fn test_search_relations_prefixed() -> Result<(), Box<dyn std::error::Erro
     // Test: prefixed search with "cat" and filters
     // --------------------------------------------------------------
 
-    let response = reader
-        .relation_search(RelationSearchRequest {
+    let response = relation_search(
+        &mut reader,
+        RelationSearchRequest {
             shard_id: shard_id.clone(),
             prefix: Some(RelationPrefixSearchRequest {
                 prefix: "cat".to_string(),
@@ -396,17 +412,19 @@ async fn test_search_relations_prefixed() -> Result<(), Box<dyn std::error::Erro
                 }],
             }),
             ..Default::default()
-        })
-        .await?;
+        },
+    )
+    .await?;
 
     let expected = HashSet::from_iter(["Cat".to_string()]);
-    assert!(response.get_ref().prefix.is_some());
-    let prefix_response = response.get_ref().prefix.as_ref().unwrap();
+    assert!(response.prefix.is_some());
+    let prefix_response = response.prefix.as_ref().unwrap();
     let results = prefix_response.nodes.iter().map(|node| node.value.to_owned()).collect::<HashSet<_>>();
     assert_eq!(results, expected);
 
-    let response = reader
-        .relation_search(RelationSearchRequest {
+    let response = relation_search(
+        &mut reader,
+        RelationSearchRequest {
             shard_id: shard_id.clone(),
             prefix: Some(RelationPrefixSearchRequest {
                 prefix: "cat".to_string(),
@@ -416,12 +434,13 @@ async fn test_search_relations_prefixed() -> Result<(), Box<dyn std::error::Erro
                 }],
             }),
             ..Default::default()
-        })
-        .await?;
+        },
+    )
+    .await?;
 
     let expected = HashSet::from_iter(["Catwoman".to_string()]);
-    assert!(response.get_ref().prefix.is_some());
-    let prefix_response = response.get_ref().prefix.as_ref().unwrap();
+    assert!(response.prefix.is_some());
+    let prefix_response = response.prefix.as_ref().unwrap();
     let results = prefix_response.nodes.iter().map(|node| node.value.to_owned()).collect::<HashSet<_>>();
     assert_eq!(results, expected);
 
@@ -429,8 +448,9 @@ async fn test_search_relations_prefixed() -> Result<(), Box<dyn std::error::Erro
     // Test: prefixed search with node filters and empty query
     // --------------------------------------------------------------
 
-    let response = reader
-        .relation_search(RelationSearchRequest {
+    let response = relation_search(
+        &mut reader,
+        RelationSearchRequest {
             shard_id: shard_id.clone(),
             prefix: Some(RelationPrefixSearchRequest {
                 prefix: String::new(),
@@ -440,12 +460,13 @@ async fn test_search_relations_prefixed() -> Result<(), Box<dyn std::error::Erro
                 }],
             }),
             ..Default::default()
-        })
-        .await?;
+        },
+    )
+    .await?;
 
     let expected = HashSet::from_iter(["Cat".to_string()]);
-    assert!(response.get_ref().prefix.is_some());
-    let prefix_response = response.get_ref().prefix.as_ref().unwrap();
+    assert!(response.prefix.is_some());
+    let prefix_response = response.prefix.as_ref().unwrap();
     let results = prefix_response.nodes.iter().map(|node| node.value.to_owned()).collect::<HashSet<_>>();
     assert_eq!(results, expected);
 
@@ -453,19 +474,21 @@ async fn test_search_relations_prefixed() -> Result<(), Box<dyn std::error::Erro
     // Test: prefixed search with "zzz" term (empty results)
     // --------------------------------------------------------------
 
-    let response = reader
-        .relation_search(RelationSearchRequest {
+    let response = relation_search(
+        &mut reader,
+        RelationSearchRequest {
             shard_id: shard_id.clone(),
             prefix: Some(RelationPrefixSearchRequest {
                 prefix: "zzz".to_string(),
                 ..Default::default()
             }),
             ..Default::default()
-        })
-        .await?;
+        },
+    )
+    .await?;
 
-    assert!(response.get_ref().prefix.is_some());
-    let prefix_response = response.get_ref().prefix.as_ref().unwrap();
+    assert!(response.prefix.is_some());
+    let prefix_response = response.prefix.as_ref().unwrap();
     let results = &prefix_response.nodes;
     assert!(results.is_empty());
 
@@ -500,8 +523,9 @@ async fn test_search_relations_neighbours() -> Result<(), Box<dyn std::error::Er
     // Test: neighbours search on existent node
     // --------------------------------------------------------------
 
-    let response = reader
-        .relation_search(RelationSearchRequest {
+    let response = relation_search(
+        &mut reader,
+        RelationSearchRequest {
             shard_id: shard_id.clone(),
             subgraph: Some(EntitiesSubgraphRequest {
                 entry_points: vec![relation_nodes.get("Swallow").unwrap().clone()],
@@ -509,23 +533,25 @@ async fn test_search_relations_neighbours() -> Result<(), Box<dyn std::error::Er
                 ..Default::default()
             }),
             ..Default::default()
-        })
-        .await?;
+        },
+    )
+    .await?;
 
     let expected = HashSet::from_iter([
         ("Poetry".to_string(), "Swallow".to_string()),
         ("Swallow".to_string(), "Animal".to_string()),
         ("Swallow".to_string(), "Fly".to_string()),
     ]);
-    let neighbour_relations = extract_relations(response.get_ref());
+    let neighbour_relations = extract_relations(&response);
     assert_eq!(neighbour_relations, expected);
 
     // --------------------------------------------------------------
     // Test: neighbours search on multiple existent nodes
     // --------------------------------------------------------------
 
-    let response = reader
-        .relation_search(RelationSearchRequest {
+    let response = relation_search(
+        &mut reader,
+        RelationSearchRequest {
             shard_id: shard_id.clone(),
             subgraph: Some(EntitiesSubgraphRequest {
                 entry_points: vec![
@@ -536,8 +562,9 @@ async fn test_search_relations_neighbours() -> Result<(), Box<dyn std::error::Er
                 ..Default::default()
             }),
             ..Default::default()
-        })
-        .await?;
+        },
+    )
+    .await?;
 
     let expected = HashSet::from_iter([
         ("Newton".to_string(), "Physics".to_string()),
@@ -545,15 +572,16 @@ async fn test_search_relations_neighbours() -> Result<(), Box<dyn std::error::Er
         ("Becquer".to_string(), "Poetry".to_string()),
         ("Joan Antoni".to_string(), "Becquer".to_string()),
     ]);
-    let neighbour_relations = extract_relations(response.get_ref());
+    let neighbour_relations = extract_relations(&response);
     assert_eq!(neighbour_relations, expected);
 
     // --------------------------------------------------------------
     // Test: neighbours search on non existent node
     // --------------------------------------------------------------
 
-    let response = reader
-        .relation_search(RelationSearchRequest {
+    let response = relation_search(
+        &mut reader,
+        RelationSearchRequest {
             shard_id: shard_id.clone(),
             subgraph: Some(EntitiesSubgraphRequest {
                 entry_points: vec![RelationNode {
@@ -565,18 +593,20 @@ async fn test_search_relations_neighbours() -> Result<(), Box<dyn std::error::Er
                 ..Default::default()
             }),
             ..Default::default()
-        })
-        .await?;
+        },
+    )
+    .await?;
 
-    let neighbours = extract_relations(response.get_ref());
+    let neighbours = extract_relations(&response);
     assert!(neighbours.is_empty());
 
     // --------------------------------------------------------------
     // Test: neighbours search with filters
     // --------------------------------------------------------------
 
-    let response = reader
-        .relation_search(RelationSearchRequest {
+    let response = relation_search(
+        &mut reader,
+        RelationSearchRequest {
             shard_id: shard_id.clone(),
             subgraph: Some(EntitiesSubgraphRequest {
                 entry_points: vec![relation_nodes.get("Poetry").unwrap().clone()],
@@ -584,12 +614,43 @@ async fn test_search_relations_neighbours() -> Result<(), Box<dyn std::error::Er
                 ..Default::default()
             }),
             ..Default::default()
-        })
-        .await?;
+        },
+    )
+    .await?;
 
     let expected = HashSet::from_iter([("Poetry".to_string(), "Swallow".to_string())]);
-    let neighbour_relations = extract_relations(response.get_ref());
+
+    // Check that the relation obtained as response has appropiate metadata
+    let subgraph = response.subgraph.clone().unwrap();
+    let first_relation = &subgraph.relations[0];
+    let metadata = first_relation.metadata.as_ref().unwrap();
+
+    assert_eq!(metadata.paragraph_id, Some("myresource/0/myresource/100-200".to_string()));
+    assert_eq!(metadata.source_start, Some(0));
+    assert_eq!(metadata.source_end, Some(10));
+    assert_eq!(metadata.to_start, Some(11));
+    assert_eq!(metadata.to_end, Some(20));
+    assert_eq!(metadata.data_augmentation_task_id, Some("mytask".to_string()));
+
+    let neighbour_relations = extract_relations(&response);
     assert!(expected.is_subset(&neighbour_relations));
 
     Ok(())
 }
+
+// NOTE: we removed RelationSearch but, as we are moving to nidx, we don't want
+// to waste time migrating tests
+async fn relation_search(
+    reader: &mut TestNodeReader,
+    request: RelationSearchRequest,
+) -> anyhow::Result<RelationSearchResponse> {
+    let request = SearchRequest {
+        shard: request.shard_id,
+        vectorset: "english".to_string(),
+        relation_prefix: request.prefix,
+        relation_subgraph: request.subgraph,
+        ..Default::default()
+    };
+    let response = reader.search(request).await?;
+    Ok(response.into_inner().relation.unwrap())
+}
