@@ -28,7 +28,7 @@ pub enum AtomClause {
     KeyPrefix(String),
     Label(String),
     KeyPrefixSet((HashSet<String>, HashSet<String>)),
-    KeyField((String, String)),
+    KeyField((String, Option<String>)),
 }
 impl AtomClause {
     pub fn label(value: String) -> AtomClause {
@@ -40,7 +40,7 @@ impl AtomClause {
     pub fn key_set(resource_set: HashSet<String>, field_set: HashSet<String>) -> AtomClause {
         Self::KeyPrefixSet((resource_set, field_set))
     }
-    pub fn key_field(field_type: String, field_name: String) -> AtomClause {
+    pub fn key_field(field_type: String, field_name: Option<String>) -> AtomClause {
         Self::KeyField((field_type, field_name))
     }
     fn run<D: DataRetriever>(&self, x: Address, retriever: &D) -> bool {
@@ -83,10 +83,18 @@ impl AtomClause {
                 // Skip resource_id, then try to parse field_type and field_name and check if they match
                 if key_parts.next().is_some() {
                     if let Some(ftype) = key_parts.next() {
-                        if let Some(fname) = key_parts.next() {
-                            let ftype_str = std::str::from_utf8(ftype).unwrap();
-                            let fname_str = std::str::from_utf8(fname).unwrap();
-                            return ftype_str == field_type && fname_str == field_name;
+                        let ftype_str = std::str::from_utf8(ftype).unwrap();
+                        let ftype_matches = ftype_str == field_type;
+
+                        if let Some(field_name) = field_name {
+                            if let Some(fname) = key_parts.next() {
+                                let fname_str = std::str::from_utf8(fname).unwrap();
+                                let fname_matches = fname_str == field_name;
+                                return ftype_matches && fname_matches;
+                            }
+                        } else {
+                            // field_type-only matches are allowed
+                            return ftype_matches;
                         }
                     }
                 }
