@@ -20,8 +20,6 @@ setup(
     package_dir={"": "src"},
     install_requires=[
         "protobuf >= 4.22.3, < 5",
-        "grpcio >= 1.44.0",
-        "grpcio-tools >= 1.44.0",
         "mypy-protobuf >= 3.4.0",
         "types-protobuf >= 4.24, < 5",
     ],
